/***********************************************
     Gauss Program for Monte Carlo Experiments 
     Reference: Testing for Stochastic Monotonicity by S. LEE, O. LINTON, and Y-J. WHANG 
     Last Edited on 24 September 2008 
***********************************************/

new;

/***********  Procedures *********************************/

proc sgn(x);   /* Sign function */
local retval;
retval = (x .> 0) - (x .< 0);
retp(retval);
endp;

proc kl(x);    /* kernel function */
local retval;
retval = (3/4)*(1 - x^2).*(abs(x) .<= 1);
retp(retval);
endp;

proc gumbel(x); /* type I extreme value distribution */
local retval;
retval = exp(-exp(-x));
retp(retval);
endp;

proc ptbg(x,b); /* asymptotic expansion of type I extreme value distribution */
local retval;
retval = exp(-exp(-x-(x.^2/(8*b^2))).*(1+(x/4*b^2)));
retp(retval);
endp;


proc(1) = kde(x,x_arg,h);   /* kernel density estimator */

@
--- Inputs ---
    x = n*d vector of observations
x_arg = k*d vector of arguments of density
    h = d*1 vector of bandwidths
--- Output ---
k*1 vector of kernel density estimates
@

local n,d,k,i,arg,tmp,fhat;

   n = rows(x);
   d = cols(x);
   k = rows(x_arg);

 tmp = ones(k,n);
   i = 1;
   do until i > d;

          arg = (x_arg[.,i]-x[.,i]')/h[i];
          tmp = tmp.*(kl(arg)/h[i]);

   i = i + 1;
   endo;

fhat = meanc(tmp');

retp(fhat);
endp;



/******* Supremum Test Statistic ************************/

proc(2) = llwtest(y,x,h,zp,bt);

@
--- Inputs ---
 y, x = data 
    h = bandwidth
   zp = grid points for x
   bt = beta (normalizing constant) 
--- Output ---
test statistics (Gumbel and Piterbarg) 
@

local yp,suptest,n,den1,den2,denmat,i_x,z,num,num1,num2,den,i,xi,i_y,t,test,sup_test,tmpstat,stat1,stat2; 

 yp = y;                                    /* observed Y in [min Y,max Y] */ 


/*--- Alternatively,
yp = seqa(minc(y),(maxc(y)-minc(y))/49,50); /* grid points in [min Y,max Y] */ 
*/


suptest = {};

/** using the asymptotic approximation **/

        n = rows(x);
     den1 = 4*59/(385*h);
     den2 = kde(x,zp,h);   
   denmat = sqrt(den1*(den2^3)/n);   



  i_x = 1;
  do until i_x .> rows(zp);
    
    z = zp[i_x];


 num2 = (kl((x-z)/h)/h)*(kl((x'-z)/h)/h);
 num2 = diagrv(num2, zeros(n,1));  @ exclude diagonal components @


/*-- Alternative way: using a triple sum

  den = 0;
  
    i = 1;
    do until i > n;
    
      xi = x[i];
    den1 = sgn(xi - x)*sgn(xi - x');
    den1 = diagrv(den1, zeros(n,1));  @ exclude diagonal components @
     
    den1 = ((kl((xi-z)/h)/h)^2)*den1;
    deni = sumc(sumc(den1.*num2));
     den = den + deni;
     
    i = i + 1;
    endo;
    
     den = 4*den/(n*(n-1)*(n-2));     
     den = den + 1e-8*(den <= 1e-8);
     den = sqrt(den/n);

--*/
 


     den = denmat[i_x];

  i_y = 1;
  do until i_y > rows(yp);
      
    t = yp[i_y]; 
    

 num1 = ((y .<= t) - (y' .<= t)).*sgn(x-x');
  num = (sumc(sumc(num1.*num2)))/(n*(n-1));

 test = num/den;



suptest = suptest~test;


i_y = i_y + 1;
 endo;

 
 i_x = i_x + 1;
 endo;
 
 sup_test = maxc(suptest');


  tmpstat = (4*bt)*(sup_test - bt);
    stat1 = gumbel(tmpstat); 
    stat2 = ptbg(tmpstat,bt);

retp(stat1,stat2);
endp;    


/***********  Main Program *********************************/


format /m1 /rd 12,5;
output file = monte_carlo.out on;


output off;

tstart = date;

hyposet = 0;  @-- set 0 for the null hypothesis; set 1 for the alternative hyphothesis --@

nset = {50 @, 100, 200, 500@};   @-- choose the sample size --@

i_nset = 1;
do until i_nset > rows(nset);

seed = 123456789;

   n = nset[i_nset];           @ sample size @
nrep = 500;                    @ number of replications @
rep_star = 500;                @ number of bootstrap samples @


bdwset = seqa(0.4,0.1,4);      @ possible bandwidths @

i_bdw = 1;
do until i_bdw > rows(bdwset);

/****** Tuning Parameters *************************/

  bdw = bdwset[i_bdw];
    h = bdw;                /* bandwidth */
   zp = seqa(0.05,0.05,19); /* X in [0.05,0.95] */    



/*-- Alternative grid search methods --
   zp = seqa(0.05,0.01,91);   /* X in [0.05,0.95] */ 
   zp = seqa(0.05,0.001,901); /* X in [0.05,0.95] */ 
*/



/******* Critical Value ************************/

  lev0 = 0.01;   @ 1% test @
  lev1 = 0.05;   @ 5% test @
  lev2 = 0.10;   @ 10% test @


   ld = 1177/118;


cstar = (8*ld/pi)^(1/2);
 tmp1 = 0.5*ln(cstar*(maxc(zp)-minc(zp))/h);
   bt = tmp1^(1/2) + ln(tmp1)/(8*tmp1^(1/2));


/* checking the graphs and quantiles of Gumbel and its approximation 
library pgraph;
tmp1 = sortc(rndn(5000,1),1);
tmp2 = ptbg(tmp1,bt);
tmp3 = gumbel(tmp1);
xy(tmp1,tmp2~tmp3);

tm = quantile(tmp1,0.95);
ptbg(tm,bt)~quantile(tmp2,0.95);
gumbel(tm)~quantile(tmp3,0.95);
*/
 

/*-- alternative way of solving the equation --
    bt_seq = seqa(bt-0.5,0.001,1000);
   zeroftn = tmp1 + 0.5*ln(bt_seq) - bt_seq^2;
   bt = bt_seq[minindc(abs(zeroftn))];
*/

cv0 = (1-lev0);
cv1 = (1-lev1);
cv2 = (1-lev2);


  result = {};
g_result = {};
     bcv = {};
   g_bcv = {};  


i_rep = 1;
do until i_rep > nrep;


/*** Data Generating Process ***/

   x = rndus(n,1,seed);
   
if hyposet == 0;     @ H_0 @
  mx = 0;
elseif hyposet == 1; @ H_1 @
  mx =  x.*(1-x); 
endif;

  sm = 0.1;
   u = sm*rndn(n,1);
   y = mx + u;

data = sortc(y~x,1);  /* sort based on y */
   y = data[.,1];
   x = data[.,2];




/******* Supremum Test Statistic ************************/

{stat1, stat2} = llwtest(y,x,h,zp,bt);

        result = result~stat2;
      g_result = g_result~stat1;




/******* Boostrap samples under independenc between X and Y ************************/

  result_star = {};
g_result_star = {};

i_star = 1;
do until i_star > rep_star;

      bsample_x = ceil(n*rndu(n,1));          @ random sampling with replacement @
         x_star = x[bsample_x];

      bsample_y = ceil(n*rndu(n,1));          @ random sampling with replacement @
         y_star = y[bsample_y];

{stat1_star, stat2_star} = llwtest(y_star,x_star,h,zp,bt);

        result_star = result_star~stat2_star;
      g_result_star = g_result_star~stat1_star;

i_star = i_star + 1;
endo;

  bcv = bcv~quantile(result_star', (cv0~cv1~cv2)');
g_bcv = g_bcv~quantile(g_result_star', (cv0~cv1~cv2)');


" Asymp: n - bdw - i_rep - rej1 - rej2 " n~bdw~i_rep~meanc(result' .>= cv1)~meanc(g_result' .>= cv1);
" Boots: n - bdw - i_rep - rej1 - rej2 " n~bdw~i_rep~meanc(result' .>= bcv[2,.]')~meanc(g_result' .>= g_bcv[2,.]');


i_rep = i_rep + 1;
endo;




/******* Rejection Proportions using Asymptotic Expansion ************************/

 rej0 = meanc(result' .>= cv0);
 rej1 = meanc(result' .>= cv1);
 rej2 = meanc(result' .>= cv2);

/******* Rejection Proportions using Asymptotic Distribution ************************/

 grej0 = meanc(g_result' .>= cv0);
 grej1 = meanc(g_result' .>= cv1);
 grej2 = meanc(g_result' .>= cv2);



/******* Bootstrap Rejection Proportions using Asymptotic Expansion ************************/

 brej0 = meanc(result' .>= bcv[1,.]');
 brej1 = meanc(result' .>= bcv[2,.]');
 brej2 = meanc(result' .>= bcv[3,.]');

/******* Bootstrap Rejection Proportions using Asymptotic Distribution ************************/

 gbrej0 = meanc(g_result' .>= g_bcv[1,.]'); 
 gbrej1 = meanc(g_result' .>= g_bcv[2,.]'); 
 gbrej2 = meanc(g_result' .>= g_bcv[3,.]'); 
    

/******* Printing Results ************************/  

  etime = ethsec(tstart,date)/6000;

output on;
"===============================================================";
if hyposet == 0;
" Monte Carlo Results: H_0 is true ";
elseif hyposet == 1;
" Monte Carlo Results: H_0 is false ";
endif;
" Sample size " n;
" Number of simulations " nrep;
" Number of bootstrap replications " rep_star;
" Bandwidth " bdw;
"===============================================================";
" Supremum  Test: Nominal Level 1% (using the asymptotic expansion) ";
" Sample Size -- Bandwidth --  Nominal Level -- Rejection Probability";
n~h~cv0~rej0;
" Supremum  Test: Nominal Level 5% ";
" Sample Size -- Bandwidth --  Nominal Level -- Rejection Probability";
n~h~cv1~rej1;
" Supremum  Test: Nominal Level 10% ";
" Sample Size -- Bandwidth --  Nominal Level -- Rejection Probability";
n~h~cv2~rej2;
"===============================================================";
" Supremum  Test: Nominal Level 1% (using the asymptotic distribution) ";
" Sample Size -- Bandwidth --  Nominal Level -- Rejection Probability";
n~h~cv0~grej0;
" Supremum  Test: Nominal Level 5% ";
" Sample Size -- Bandwidth --  Nominal Level -- Rejection Probability";
n~h~cv1~grej1;
" Supremum  Test: Nominal Level 10% ";
" Sample Size -- Bandwidth --  Nominal Level -- Rejection Probability";
n~h~cv2~grej2;
"===============================================================";
" Supremum  Test: Nominal Level 1% (Bootstrap: using the asymptotic expansion) ";
" Sample Size -- Bandwidth --  Nominal Level -- Rejection Probability";
n~h~cv0~brej0;
" Supremum  Test: Nominal Level 5% ";
" Sample Size -- Bandwidth --  Nominal Level -- Rejection Probability";
n~h~cv1~brej1;
" Supremum  Test: Nominal Level 10% ";
" Sample Size -- Bandwidth --  Nominal Level -- Rejection Probability";
n~h~cv2~brej2;
"===============================================================";
" Supremum  Test: Nominal Level 1% (Bootstap: using the asymptotic distribution) ";
" Sample Size -- Bandwidth --  Nominal Level -- Rejection Probability";
n~h~cv0~gbrej0;
" Supremum  Test: Nominal Level 5% ";
" Sample Size -- Bandwidth --  Nominal Level -- Rejection Probability";
n~h~cv1~gbrej1;
" Supremum  Test: Nominal Level 10% ";
" Sample Size -- Bandwidth --  Nominal Level -- Rejection Probability";
n~h~cv2~gbrej2;
"===============================================================";
output off;

i_bdw = i_bdw + 1;
endo;

i_nset = i_nset + 1;
endo;

output on;
" Running time   " etime;
"";
 output off; 
 
end;
